/*
 * Copyright (c) 2020.  Anton Romanov
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ulstu.controllers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.HttpServerErrorException;
import ru.ulstu.models.exceptions.ForecastValidateException;
import ru.ulstu.models.exceptions.TimeSeriesValidateException;
import ru.ulstu.models.response.ErrorConstants;
import ru.ulstu.models.response.ResponseExtended;

@RestController
@ControllerAdvice
public class AdviceController {
    private final Logger log = LoggerFactory.getLogger(AdviceController.class);

    private <E> ResponseExtended<E> handleException(ErrorConstants error, E errorData) {
        log.warn(error.toString());
        return new ResponseExtended<>(error, errorData);
    }

    @ExceptionHandler(Exception.class)
    public ResponseExtended<String> handleUnknownException(Throwable e) {
        e.printStackTrace();
        return handleException(ErrorConstants.UNKNOWN, e.getMessage());
    }

    @ExceptionHandler(HttpServerErrorException.class)
    public ResponseExtended<String> handleHttpClientException(Throwable e) {
        return handleException(ErrorConstants.HTTP_CLIENT_ERROR, e.getMessage());
    }

    @ExceptionHandler(TimeSeriesValidateException.class)
    public ResponseExtended<String> handleTimeSeriesValidateException(Throwable e) {
        return handleException(ErrorConstants.TIME_SERIES_VALIDATE_ERROR, e.getMessage());
    }

    @ExceptionHandler(ForecastValidateException.class)
    public ResponseExtended<String> handleForecastValidateException(Throwable e) {
        return handleException(ErrorConstants.FORECAST_PARAMS_ERROR, e.getMessage());
    }
}
