package ru.ulstu.controllers;

import io.swagger.annotations.ApiOperation;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import ru.ulstu.configurations.ApiConfiguration;
import ru.ulstu.models.Forecast;
import ru.ulstu.models.ForecastParams;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.ModelingException;
import ru.ulstu.services.TimeSeriesService;

@RestController
@RequestMapping(ApiConfiguration.API_1_0)
public class TimeSeriesController {

    private final TimeSeriesService timeSeriesService;

    public TimeSeriesController(TimeSeriesService timeSeriesService) {
        this.timeSeriesService = timeSeriesService;
    }

    @GetMapping("isAlive")
    @ApiOperation("Проверка сервиса")
    public ResponseEntity<Boolean> isAlive() {
        return new ResponseEntity<>(true, HttpStatus.OK);
    }

    @GetMapping("getRandom")
    @ApiOperation("Получить временной ряд рандомной длины")
    public ResponseEntity<TimeSeries> getRandomTimeSeries(@RequestParam("length") int length) {
        return new ResponseEntity<>(timeSeriesService.getRandomTimeSeries(length), HttpStatus.OK);
    }

    @GetMapping("getFromString")
    @ApiOperation("Преобразовать строку с разделителями во временной ряд")
    public ResponseEntity<TimeSeries> getTimeSeriesFromString(@RequestParam("tsString") String tsString) {
        return new ResponseEntity<>(timeSeriesService.getTimeSeriesFromString(tsString), HttpStatus.OK);
    }

    @PostMapping("timeSeriesToString")
    @ApiOperation("Преобразовать временной ряд в строку с разделителями")
    public ResponseEntity<String> getTimeSeriesToString(@RequestBody TimeSeries timeSeries) {
        return new ResponseEntity<>(timeSeriesService.getTimeSeriesToString(timeSeries), HttpStatus.OK);
    }

    @PostMapping("getForecast")
    @ApiOperation("Получить прогноз временного ряда")
    public ResponseEntity<Forecast> getForecastTimeSeries(@RequestBody ForecastParams forecastParams) throws ModelingException {
        return new ResponseEntity<>(timeSeriesService.getForecast(forecastParams.getOriginalTimeSeries(),
                forecastParams.getCountForecast()), HttpStatus.OK);
    }
}
